//
//  GLMapController.h
//  GLMapKit
//
//  Created by Jeffrey Early on 10/1/06.
//  Copyright 2006 Early Innovations, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryKit.h>

extern NSString *GLLongitudeCoordinateDefault;
extern NSString *GLLatitudeCoordinateDefault;
extern NSString *GLScaleDefault;
extern NSString *GLSelectedMapDefault;

@class GLMapView, GLMapDataSource, GLMapTool, GLMapBackForwardList;

@interface GLMapController : NSObject <NSTableViewDataSource> {
	// The two primary objects of mediation
	IBOutlet GLMapView *mapView;
	id spatialObjectController;
    __unsafe_unretained id delegate;
    __unsafe_unretained id draggingDestinationDelegate;
	
    GLPoint locationOfDroppedItem;
    
	NSMutableArray	*spatialObjectControllers;
	NSArrayController	*spatialObjectsArrayController;
	IBOutlet NSTableView *spatialObjectControllerTableView;
	NSLock *spatialObjectLock;
	
	NSTextField *licenseTextField;
	
	// For the data source popup
	NSMutableArray *mapDataSourceArray;
	NSUInteger mapDataSourceSelectedIndex;
	NSString *mapProjectionName;
	
	double maximumFitResolution;
	
	GLPoint latitudeLongitude;
	double scale;
	double minSliderScale;
	double maxSliderScale;
	double sliderScale;
	
	GLMapBackForwardList *mapBackFowardList;
}

//=========================================================== 
//  MapView 
//=========================================================== 

#pragma mark -
#pragma mark MapView
#pragma mark

// Return the MapView instance being controlled by the controller/
// The MapView *needs* to be connected in interface builder.
@property(readonly) GLMapView * mapView;

//=========================================================== 
//  MapDataSource 
//=========================================================== 

#pragma mark -
#pragma mark MapDataSource
#pragma mark

// Returns an array of of instantiated GLMapDataSource classes.
// GLMapDataSource responds to - (NSString *) name;
@property(readonly) NSMutableArray * mapDataSourceArray;

// Selection index of the current datasource
@property(nonatomic) NSUInteger mapDataSourceSelectedIndex;

// Returns the current data source. Useful for determining the number of current downloads.
@property(readonly) GLMapDataSource * mapDataSource;

// Returns the map projection currently employed.
@property(copy) NSString *mapProjectionName;

//=========================================================== 
//  Latitude & Longitude 
//===========================================================

#pragma mark -
#pragma mark Latitude & Longitude
#pragma mark

// This functions all change the position and/or scale of the map in one way or another.

// Sent latitude, longitude in WGS84 with a scale in meters per pixel
- (void) setLatitudeLongitude: (GLPoint) latLong withScale: (double) aScale;

// Latitude and longitude in WGS84
@property(nonatomic) GLPoint latitudeLongitude;

// KVO compatible, for bindings
@property(readonly) double latitude;
@property(readonly) double longitude;

// Enable center of map crosshairs
@property BOOL shouldShowCrosshairs;

// This is an alternative to set the lat/lon and scale. Here, you actually set a desired
// bounding box (in lat/lon) and the controller will zoom to the appropriate location
// and scale to fit the entire bounding box, when possible.
- (void) setMinBoundingRect: (GLRect) bounds;

// This function performs the minimum amount of zooming and panning necessary to show
// the spatial objects. So if the objects are already displayed, no action will be performed.
- (void) revealSpatialObjects: (NSArray *) aSpatialObjectsArray withAnimation: (BOOL) animate;
- (void) revealSpatialObjects: (NSArray *) spatialObjects;

// This function simply calls the setMinBoundingRect for the union rect of the functions.
- (void) fitSpatialObjects: (NSArray *) aSpatialObjectsArray withAnimation: (BOOL) animate;
- (void) fitSpatialObjects: (NSArray *) spatialObjects;

// Sets a maximum resolution that the -fitSpatialObjects: function will use. Default 2 meters/pixel.
@property double maximumFitResolution;

// These zoom the map center one level.
- (void) zoomIn;
- (void) zoomOut;

//=========================================================== 
//  Scale 
//=========================================================== 

#pragma mark -
#pragma mark Scale
#pragma mark

// All basic scale methods are in meters per pixel and can be set as indicated.
// However, an interface will not want to bind to these scale values as they can
// change each pixel the user scrolls around. Some maps do not have a constant
// meter/pixel ratio. Please use sliderScale values instead: a meaningless, but
// constant scale value.

// Scale in meters per pixel
@property(nonatomic) double scale;

// return minimum scale allowed for the currently selected datasource
@property(readonly) double minScale;

// return maximum scale allowed for the currently selected datasource
@property(readonly) double maxScale;

// Scale in sliderScale units
@property(nonatomic) double sliderScale;

// return minimum scale allowed for the currently selected datasource
@property(nonatomic) double minSliderScale;

// return maximum scale allowed for the currently selected datasource
@property(nonatomic) double maxSliderScale;

//=========================================================== 
//  Delegate 
//=========================================================== 

#pragma mark -
#pragma mark Delegate
#pragma mark

// The delegate must implement GLMapDelegate protocol
@property(unsafe_unretained) id delegate;

// The delegate (NSDraggingDestination) to set.
@property(unsafe_unretained) id draggingDestinationDelegate;

// Returns the location of the last dropped item.
@property GLPoint locationOfDroppedItem;

//=========================================================== 
//  Map Back-Forward List
//=========================================================== 

#pragma mark -
#pragma mark Map Back-Forward List
#pragma mark

// Page forward or backward to locations/scales.

@property(readonly) GLMapBackForwardList *mapBackFowardList;

- (void) goBack;
- (void) goForward;

//=========================================================== 
//  SpatialObjectController 
//=========================================================== 

#pragma mark -
#pragma mark SpatialObjectController
#pragma mark

// Add or remove spatial object controllers
- (void) addSpatialObjectController: (id) newController;
- (void) removeSpatialObjectController: (id) oldController;

@property(readwrite, strong) NSMutableArray *spatialObjectControllers;
@property(readwrite, strong) NSMutableIndexSet *spatialObjectSelectionIndexes;
@property(readwrite, strong) NSMutableArray *spatialObjects;

// Changing the selection should be done by the individual controllers
// These methods are used internally by the mapView.
- (void) setNewSpatialObjectSelectionIndexes: (NSIndexSet *) indexSet;
- (void) addSelectedSpatialObjects: (NSArray *) objects;
- (void) removeSelectedSpatialObjects: (NSArray *) objects;

//=========================================================== 
//  MapTools 
//===========================================================

#pragma mark -
#pragma mark MapTools
#pragma mark

// One can register for GLSelectedMapToolDidChangeNotification.
// Array of valid, instantiated map tools (browse, select, etc.)
- (NSArray *) mapToolArray;

// Index of the selected map tool
- (int) mapToolSelectedIndex;

// Set the index of the selected map tool
- (void) setMapToolSelectedIndex: (int) aValue;

// Alternative method for determining the selected map tool
- (GLMapTool *) selectedMapTool;

// Pass one of the map tools
- (void) setSelectedMapTool: (GLMapTool *) mapTool;

//=========================================================== 
//  Defaults 
//=========================================================== 
// Set the default lat/lon, scale, maps

@property(strong) GLMapDataSource * defaultMapDataSource;
@property GLPoint defaultLatitudeLongitude;
@property double defaultScale;

@end

/************************************************/
/*		GLMapDelegate							*/
/************************************************/

#pragma mark -
#pragma mark GLMapDelegate
#pragma mark

@interface NSObject (GLMapDelegate)
-(void) mapView: (GLMapView *) aMapView wasRightClickedAtLocation: (GLPoint) latLonPoint withEvent: (NSEvent *) anEvent;
@end
